import {Route, Routes, useNavigate} from 'react-router';
import {Layout} from "./components/Layout.tsx";
import {PaneMain} from "./routes/PaneMain.tsx";
import {PaneMenu} from "./routes/PaneMenu.tsx";
import {WebSocketHandler} from "./utils/WebSocketHandler.ts";
import {useEffect, useRef, useState} from "react";
import {iTenderStatus} from "./interfaces/iTenderStatus.ts";
import {Pane} from "./interfaces/Pane.ts";
import {PaneStartup} from "./routes/PaneStartup.tsx";
import {WebSocketEvent} from "./interfaces/WebSocketEvent.ts";

function App() {
    const navigate = useNavigate();
    const [currentStatus, setCurrentStatus] = useState<iTenderStatus>(iTenderStatus.CONNECTING);
    const [currentPane, setCurrentPane] = useState<Pane>(Pane.NONE);
    const cleanupHook = useRef<undefined | (() => void)>(undefined);

    const connect = () => {
        console.log("[App.tsx] Connecting...");
        WebSocketHandler.connect(() => {
            // now open
            if (cleanupHook.current)
                cleanupHook.current();
            cleanupHook.current = WebSocketHandler.registerForEvent(WebSocketEvent.STATUS, (payload) => {
                setCurrentStatus(payload.data.status);
            });

            /*console.log("setting pane main");
            setCurrentPane(Pane.MAIN);*/
        }, (wasClean: boolean) => {
            setCurrentStatus(iTenderStatus.CONNECTING);
            if (wasClean) {
                // loop???????????????????
                console.log("[App] WS was closed clean, trying to reconnect in 1s");
                setTimeout(connect, 1000);
            }
        });
    }

    useEffect(() => {
        connect();

    }, []);

    useEffect(() => {
        switch (currentStatus) {
            case iTenderStatus.CONNECTING:
                setCurrentPane(Pane.NONE);
                break;
            case iTenderStatus.READY:
                setCurrentPane(Pane.MAIN);
                break;
        }
    }, [currentStatus]);

    useEffect(() => {
        console.log("Changing pane", currentPane);
        if (currentPane == Pane.NONE) {
            setCurrentStatus(iTenderStatus.STARTING);
            navigate("/");
        } else
            navigate("panes/" + currentPane);
    }, [currentPane]);


    return (
        <Routes>
            <Route path={"/"} element={<Layout/>}>
                <Route index path="/" element={<PaneStartup onRetryClick={() => connect()}/>}/>
            </Route>
            <Route path="/panes" element={<Layout/>}>
                <Route path={"/panes/paneMain"} element={<PaneMain/>}/>
                <Route path="/panes/paneMenu" element={<PaneMenu/>}/>
            </Route>
        </Routes>)
}

export default App
