import {Outlet} from "react-router";
import {useEffect, useState} from "react";
import {WebSocketHandler} from "../utils/WebSocketHandler.ts";
import {WebSocketEvent} from "../interfaces/WebSocketEvent.ts";
import IContainer from "../interfaces/db/IContainer.ts";


export function Layout() {
    const [currentTime, setCurrentTime] = useState("00:00");
    const [status, setStatus] = useState("Booting up...");
    const [containers, setContainers] = useState<IContainer[]>([]);

    useEffect(() => {
        console.log("hooks applied")
        let currentTimeInterval = setInterval(() => {
            let date = new Date();
            setCurrentTime((date.getHours() < 10 ? "0" + date.getHours() : date.getHours()) + ":" + (date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes()));
        }, 1000);
        const statusHook = WebSocketHandler.registerForEvent(WebSocketEvent.STATUS, (payload) => {
            console.log(payload);
            setStatus(payload.data.status);
        });
        const containerHook = WebSocketHandler.registerForEvent(WebSocketEvent.CONTAINERS, (payload) => {
            setContainers(payload.data);
        });

        return () => {
            clearInterval(currentTimeInterval);
            statusHook();
            containerHook();
        }
    }, []);

    return (
        <>
            <div id={"overlay"}>
                <div id={"top"}>
                    <span id="left">
                        <strong>Status: </strong>
                        <span id="status">{status}</span>
                    </span>
                    <span id="title">iTender</span>
                    <span id="right">{currentTime}</span>
                </div>
                <div id={"bottom"}>
                    <div>
                        <button id={"menuBtn"} className={"btn"}>Menü</button>
                    </div>

                    <div id={"menuContainers"}>
                        {containers.map((container) => (
                            <div key={container._id} className={"container"}>
                                <div className={"tooltip"}>
                                    {!container.content ? "-" : Math.round(container.filled * 100 / container.volume)}%
                                    <span
                                        className={"tooltiptext"}>{container.content ? container.content.name : "-"} {Math.ceil(container.filled)}ml</span>
                                </div>
                                <span className={"tooltiptext"}></span>
                            </div>
                        ))}
                        {/*<div className={"container"}>
                            <span>3%</span>
                        </div>*/}
                    </div>
                </div>
            </div>
            <div id={"pane_container"}>
                <Outlet/>
            </div>
        </>
    )
}