import {WebSocketEvent} from "./WebSocketEvent.ts";

export class WebSocketPayload<T> {
    set event(value: WebSocketEvent) {
        this._event = value;
    }


    set data(value: T) {
        this._data = value;
    }

    private _event: WebSocketEvent;
    private _data: T;


    get event(): WebSocketEvent {
        return this._event;
    }



    get data(): T {
        return this._data;
    }

    constructor(event: WebSocketEvent, data?: any) {
        this._event = event;
        this._data = data;
    }

    public static parseFromBase64Json(json: any): WebSocketPayload<any> | null {
        //json = (typeof window != 'undefined') ? atob(json).toString() : Buffer.from(json, "base64").toString("utf-8");
        json = atob(json);
        let rawPayload: { event: string, data: any };
        try {
            rawPayload = JSON.parse(json);
        } catch {
            return null;
        }
        let wsEvent = WebSocketEvent[<keyof typeof WebSocketEvent>rawPayload.event];

        return new WebSocketPayload(wsEvent, rawPayload.data);
    }

    /**
     * Returns the payload as base64 encoded json string
     */
    public toString(): string {
        let json = JSON.stringify({"event": this._event, data: this._data});

        json = ((typeof window != 'undefined') ? btoa(json) : Buffer.from(json).toString("base64"));
        return json;
    }
}